class PanelPicture < Peta::Element
  load_manifest
  belongs_to :panel
  belongs_to :picture
  
  validates :panel_id, :numericality => {:allow_blank => true}
  validates :picture_id, :numericality => true, :existence => {:both => false}
  validates :link, :length => {:maximum => 200}, :allow_blank => true
  #validates :link, :length => {:maximum => 200}, :url => {:allow_blank => true, :message => I18n.t('errors.messages.url')}
  validates :x, :presence => true, :numericality => true
  validates :y, :presence => true, :numericality => true
  validates :width, :presence => true, :numericality => true, :not_zero => true, :reverse => true, :sync_vh => true
  validates :height, :presence => true, :numericality => true, :not_zero => true, :reverse => true, :sync_vh => true
  validates :z, :presence => true, :numericality => {:greater_than => 0}
  validates :t, :presence => true, :numericality => {:greater_than_or_equal_to => 0}
  
  scope :with_panel, -> do
    includes(:panel)
  end
  
  scope :find_index, -> do
    with_panel.where(Panel.arel_table[:publish].gt 0).references(:panel)
  end
  
  scope :find_private, -> (operators) do 
    with_panel.where(Panel.arel_table[:author_id].eq operators.author.id).references(:panel)
  end
  
  scope :find_by_panel, -> (panel_id) do 
    find_index.where(panel_id: panel_id).references(:panel)
  end
  
  scope :find_by_author, -> (author_id) do 
    find_index.where(Panel.arel_table[:author_id].eq author_id).references(:panel)
  end
  
  def self.pickup_item_name
    Picture.item_name
  end
  
  def self.pickup_column_name
    self.pickup_item_name + '_id'
  end
  
  def pickup_id
    # get :picture_id if head revision
    self.attributes[self.pickup_column_name]
  end
  
  def y
    self.attributes['y']
  end
  
  def self.by_author_list_includes
    {
      :panel => {
        :author => {}
      }
    }
  end
  
  def self.has_picture?
    true
  end
  
  def supply_default
    self.x = 0
    self.y = 0
    if self.picture
      self.width = self.picture.width 
      self.height = self.picture.height 
    end
    if self.panel
      self.t = self.panel.new_t 
      self.z = self.panel.new_z 
    end
  end
  
  def overwrite  pid
    self.panel_id = pid
  end
  
  def flip
    res = (self.height > 0 ? '' : 'v') + (self.width > 0 ? '' : 'h')
    res = res # format of /1.png?subdir=v
    # res += '/' unless res.empty? # format of /v/1.png
    res
  end
  
  def filename
    q = self.flip.empty? ? '' : '?subdir=' + self.flip
    self.picture.filename + q
  end
  
  def url
    '/pictures/' + self.filename
  end
  
  def opt_div_style
    "top:#{self.y}px; left:#{self.x}px; z-index:#{self.z}; position: absolute;"
  end
  
  def opt_img_tag spot = nil, opacity = 20
    o = (spot and spot != self) ? "opacity: #{opacity.to_f/100}; filter:alpha(opacity=#{opacity});" : ''
    self.tag_attributes(:img, {
      :class => "panel-picture", :vPicture => self.id, 
      :src => self.url, :width => self.width.abs, :height => self.height.abs, :picture_id => self.picture_id, :ext => self.picture.ext, :alt => self.caption, 
      :style => "#{o}"
    })
  end
  
  def tmb_opt_img_tag
    tw, th = PettanImager.thumbnail_size(self.width.abs, self.height.abs)
    {:src => self.url, :width => tw, :height => th, :alt => self.caption}
  end
  
  def symbol_option
    self.picture.symbol_option
  end
  
  def self.public_list_where list
    'panels.publish > 0'
  end
  
  def self.show_opt
    {:include => {:panel => {:author => {}}, :picture => {:artist => {}, :license => {}}}}
  end
  
  def disp_link
    if self.link =~ /\:\/\//
      self.link
    else
      '/' + self.link
    end
  end
  
  def scenario
    if caption.blank?
      ''
    else
      '<p>' + ERB::Util.html_escape(self.caption) + '</p>'
    end
  end
  
  def plain_scenario
    if caption.blank?
      ''
    else
      self.caption + "\n"
    end
  end
  
end
