class SheetPanel < Peta::Element
  load_manifest
  belongs_to :panel
  belongs_to :sheet
  accepts_nested_attributes_for :panel, :allow_destroy => true
  #alias_attribute :attr_y, :y
  
  validates :sheet_id, :numericality => {:allow_blank => true}
  validates :panel_id, :numericality => {:allow_blank => true}
  validates :x, :presence => true, :numericality => true
  validates :y, :presence => true, :numericality => true
  validates :z, :presence => true, :numericality => {:greater_than => 0}
  validates :t, :presence => true, :numericality => {:greater_than_or_equal_to => 0}
  
  scope :with_sheet, -> do
    includes(:sheet)
  end
  
  scope :find_index, -> do
    with_sheet.where(Sheet.arel_table[:visible].gt 0).references(:sheet)
  end
  
  scope :find_private, -> (operators) do 
    with_sheet.where(Sheet.arel_table[:author_id].eq operators.author.id).references(:sheet)
  end
  
  scope :by_sheet, -> (sheet_id) do 
    where(sheet_id: sheet_id)
  end
  
  scope :find_by_sheet, -> (sheet_id) do 
    find_index.by_sheet(sheet_id).references(:sheet)
  end
  
  scope :find_by_panel, -> (panel_id) do 
    find_index.where(panel_id: panel_id).references(:sheet)
  end
  
  scope :find_by_author, -> (author_id) do 
    find_index.where(Sheet.arel_table[:author_id].eq author_id).references(:sheet)
  end
  
  scope :find_play, -> (sheet_id) do 
    find_by_sheet(sheet_id)
  end
  
  scope :find_private_play, -> (sheet_id, operators) do 
    find_private(operators).by_sheet(sheet_id)
  end
  
  def y
    self.attributes['y']
  end
  
  def supply_default
    self.x = 15
    self.y = 15
    if self.sheet
      self.t = self.sheet.new_t 
      self.z = self.sheet.new_z 
    else
      self.sheet_id = nil
      self.panel_id = nil
      self.z = 1
      self.t = nil
    end
  end
  
  def overwrite
  end
  
  def self.by_author_list_includes
    {
      :sheet => {
        :author => {}
      }
    }
  end
  
  def self.show_opt
    {:include => {
      :sheet => {
        :author => {}
      }, 
      :panel => {
        :author => {}, 
        :panel_pictures => {:picture => {:artist => {}, :license => {}}}, 
        :speech_balloons =>{:balloon => {}, :speech => {}}
      }
    }}
  end
  
  def elements
    self.panel.elements
  end
  
  def sheet_panel_as_json au
    panel_include = if self.panel and self.panel.visible?(au)
      {:include => {}, :methods => :elements}
    else
      {:include => {}}
    end
    self.to_json({:include => {:sheet => {:include => {:author => {}}}, :panel => panel_include}})
  end
  
  def self.list_as_json_text ary, au
    '[' + ary.map {|i| i.sheet_panel_as_json(au) }.join(',') + ']'
  end
  
  def licensed_pictures
    if self.panel
      self.panel.licensed_pictures
    else
      {}
    end
  end
  
  def copyable?
    if self.panel and self.panel.publish?
      true
    else
      false
    end
  end
  
  def allow? operators
    return nil if self.sheet_id == nil or self.panel_id == nil
    self.sheet.own?(operators) and self.panel.usable?(operators)
  end
  
  def store operators, old_t = nil
    res = false
    self.class.transaction do
      case self.allow? operators
      when true
        self.rotate old_t
      when false
        raise ActiveRecord::Forbidden
      else
      end
      res = self.save
      raise ActiveRecord::Rollback unless res
      res = self.class.validate_t(self.sheet_id) 
      unless res
        self.errors.add :t, 'unserialized'
        raise ActiveRecord::Rollback 
      end
    end
    res
  end
  
end
