module Manifest
  module ControllerModule
    module ActionModule
      class ActionList < Base
        attr :order, :direction, :default_page_size, :max_page_size, :return_item_name
        
        def set_default
          super
          @args['direction'] ||= -1
          @args['default_page_size'] ||= 25
          @args['max_page_size'] ||= 100
        end
        
        def init
          super
          @order = @args['order'] || 'updated_at'
          @direction = @args['direction'].to_i
          @default_page_size = @args['default_page_size']
          @max_page_size = @args['max_page_size']
          @return_item_name = @args['return_item_name'] || @item_name
        end
        
        def a_arg_names
          super + ['order', 'direction', 'default_page_size', 'max_page_size']
        end
        
        def find options = {}
          if has_alias_action
            return self.alias_action.find options
          end
          arel = self.finder(options).order(
            self.order_sql(options[:order], options[:direction])
          )
          if (ps = self.page_size(options[:page_size])) or options[:page]
            arel = arel.page(self.page(options[:page])).per(ps)
          end
          arel
        end
        
        def finder options = {}
          finder_name = options[:finder] || 'find_' + @name
          finder = if args = options[:param]
            args = [args] unless options[:param].is_a?(Array)  # param to array
            self.finder_model.__send__(finder_name, *args)
          else
            self.finder_model.__send__(finder_name)
          end
        end
        
        def finder_model
          Manifest.manifest.models[@item_name].classify
        end
        
        def return_model
          Manifest.manifest.models[@return_item_name].classify
        end
        
        def order_sql order, direction
          order ||= self.order
          direction ||= (self.direction < 0 ? :desc : :asc)
          self.return_model.arel_table[order].__send__(direction)
        end
        
        def page page
          pg = page.to_i
          pg = 1 if pg < 1
          pg
        end
        
        def page_size page_size
          ps = (page_size || self.default_page_size).to_i
          return nil if ps < 0
          ps = self.max_page_size if ps > self.max_page_size
          ps
        end
        
        def limited?
          self.max_page_size > 0
        end
        
        def unlimited?
          !self.limited?
        end
        
      end
      
    end
  end
end
