# -*- encoding: utf-8 -*-
require 'spec_helper'
# コミスト

describe ComicStory do
  include ManifestsMacros
  before do
    @model = ComicStory
  end
  
  describe 'ペタナイズに於いて' do
    it 'コミックにストーリーを差し込むためのリーフである' do
      expect(@model.ancestors.include?(Peta::Leaf)).to eq true
    end
    it '親モデルはコミックである' do
      expect(@model.my_peta.parent_model_name).to eq 'comic'
    end
    describe 'ブーストに於いて' do
      it '特筆すべき注意事項はない' do
        expect(@model.my_peta.boost.keys).to be_empty
      end
    end
    
  end
  
  describe 'コントローラに於いて' do
    before do
      @controller = @model.my_controller
    end
    it 'コミストのためのコントローラである' do
      expect(@model.item_name).to eq 'comic_story'
    end
    describe 'actionに於いて' do
      before do
        @actions = @controller.actions
        @actions_names = ['index', 'by_comic', 'by_story', 'show', 
          'count', 'count_by_comic', 'count_by_story', 
          'new', 'edit', 'create', 'update', 'destroy']
      end
      it 'これらのアクションが定義されている' do
        set_actions? @actions, @actions_names
      end
      describe 'baseに於いて' do
      end
      describe 'indexに於いて' do
        before do
          @action = @actions['index']
        end
        it 'listタイプのアクションである' do
          expect(@action.type).to eq 'list'
        end
        it 'マニフェストから設定を取り出している' do
          # コミストの公開リストを返すように定義している
          set_list_action? @action, 'comic_story', 'public'
        end
      end
      describe 'by_comicに於いて' do
        before do
          @action = @actions['by_comic']
        end
        it 'listタイプのアクションである' do
          expect(@action.type).to eq 'list'
        end
        it 'マニフェストから設定を取り出している' do
          # コミストの作家フィルタリストを返すように定義している
          set_list_action? @action, 'comic_story', 'by_comic'
        end
      end
      describe 'by_storyに於いて' do
        before do
          @action = @actions['by_story']
        end
        it 'listタイプのアクションである' do
          expect(@action.type).to eq 'list'
        end
        it 'マニフェストから設定を取り出している' do
          #コミストのコマフィルタリストを返すように定義している
          set_list_action? @action, 'comic_story', 'by_story'
        end
      end
      describe 'showに於いて' do
        before do
          @action = @actions['show']
        end
        it 'showタイプのアクションである' do
          expect(@action.type).to eq 'show'
        end
      end
      describe 'countに於いて' do
        before do
          @action = @actions['count']
        end
        it 'countタイプのアクションである' do
          expect(@action.type).to eq 'count'
        end
        it 'マニフェストから設定を取り出している' do
          #コミストの公開リストのカウントを返すように定義している
          set_list_action? @action, 'comic_story', 'public'
        end
      end
      describe 'count_by_comicに於いて' do
        before do
          @action = @actions['count_by_comic']
        end
        it 'countタイプのアクションである' do
          expect(@action.type).to eq 'count'
        end
        it 'マニフェストから設定を取り出している' do
          # コミストの作家フィルタリストのカウントを返すように定義している
          set_list_action? @action, 'comic_story', 'by_comic'
        end
      end
      describe 'count_by_storyに於いて' do
        before do
          @action = @actions['count_by_story']
        end
        it 'countタイプのアクションである' do
          expect(@action.type).to eq 'count'
        end
        it 'マニフェストから設定を取り出している' do
          # コミストのコマフィルタリストのカウントを返すように定義している
          set_list_action? @action, 'comic_story', 'by_story'
        end
      end
      describe 'newに於いて' do
        before do
          @action = @actions['new']
        end
        it 'newタイプのアクションである' do
          expect(@action.type).to eq 'new'
        end
        it 'type' do
          expect(@action.item_name).to eq 'comic_story'
        end
      end
      describe 'editに於いて' do
        before do
          @action = @actions['edit']
        end
        it 'editタイプのアクションである' do
          expect(@action.type).to eq 'edit'
        end
        it 'type' do
          expect(@action.item_name).to eq 'comic_story'
        end
      end
      describe 'createに於いて' do
        before do
          @action = @actions['create']
        end
        it 'createタイプのアクションである' do
          expect(@action.type).to eq 'create'
        end
        it 'type' do
          expect(@action.item_name).to eq 'comic_story'
        end
      end
      describe 'updateに於いて' do
        before do
          @action = @actions['update']
        end
        it 'updateタイプのアクションである' do
          expect(@action.type).to eq 'update'
        end
        it 'type' do
          expect(@action.item_name).to eq 'comic_story'
        end
      end
      describe 'destroyに於いて' do
        before do
          @action = @actions['destroy']
        end
        it 'destroyタイプのアクションである' do
          expect(@action.type).to eq 'destroy'
        end
        it 'type' do
          expect(@action.item_name).to eq 'comic_story'
        end
      end
    end
    
  end
  
  describe 'モデルに於いて' do
    before do
      @my_manifest = @model.my_manifest
    end
    it 'サポート機能が動作している' do
      expect(@my_manifest.model_name).to eq 'comic_story'
      expect(@my_manifest.classify).to eq @model
      expect(@my_manifest.table_name).to eq 'comic_stories'
    end
    
    describe 'associationsに於いて' do
      describe 'belongs_toに於いて' do
        before do
          @belongs_to = @my_manifest.associations.belongs_to
          @names = ['comic', 'story']
        end
        it 'これらのモデルに所属している' do
          set_associations? @belongs_to, @names
        end
        describe 'comicに於いて' do
          before do
            @mani = @belongs_to['comic']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名と外部キーが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'comic'
            expect(@mani.id_column).to eq 'comic_id'
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている
            expect(@mani.model).to eq Comic
          end
        end
        describe 'storyに於いて' do
          before do
            @mani = @belongs_to['story']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名と外部キーが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'story'
            expect(@mani.id_column).to eq 'story_id'
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている
            expect(@mani.model).to eq Story
          end
        end
      end
      describe 'has_manyに於いて' do
        before do
          @has_many = @model.my_manifest.associations.has_many
          @names = []
        end
        it 'これらのモデルを所持している' do
          set_associations? @has_many, @names
        end
      end
    end
    
    describe 'attributesに於いて' do
      before do
        @attributes = @model.my_manifest.attributes
      end
      it 'これらのカラムを定義している' do
        set_attributes? @attributes, 
          ['id', 'comic_id', 'story_id', 't', 
          'created_at', 'updated_at']
      end
      describe 'comic_idに於いて' do
        before do
          @attribute = @attributes['comic_id']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'story_idに於いて' do
        before do
          @attribute = @attributes['story_id']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'tに於いて' do
        before do
          @attribute = @attributes['t']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
    end
    
  end
  
end
