# -*- encoding: utf-8 -*-
require 'spec_helper'
#スクロール

describe ScrollPanel do
  include LeafMacros
  before do
    @admin =FactoryGirl.create :admin
    @user_yas = FactoryGirl.create :user_yas_with_owner
    @user_rom = FactoryGirl.create :user_rom
    @author_yas =  @user_yas.author
    
    @model = ScrollPanel
    @scroll =  FactoryGirl.create :scroll_hello, author: @author_yas
    @panel =  FactoryGirl.create :panel_hello, author: @author_yas
    @item =  FactoryGirl.create :scroll_panel_plain, scroll: @scroll, panel: @panel
  end
  
  describe '生成メソッドに於いて' do
    before do
    end
    
    it '親モデルであるバインダーを取得している' do
      expect(@model.parent_model).to eq Scroll
    end
    it '差し込むコンテンツのモデルを取得している' do
      expect(@model.destination_model).to eq Panel
    end
    
    it 'バインダーを取得するためのIDのカラムを返している' do
      expect(@model.binder_key).to eq 'scroll_id'
    end
    
    it '差し込むコンテンツを取得するためのIDのカラムを返している' do
      expect(@model.destination_key).to eq 'panel_id'
    end
    
  end
  
  describe '便利機能に於いて' do
    before do
    end
    
    it 'バインダーのIDを返している' do
      expect(@item.binder_id).to eq @scroll.id
    end
    
    it '差し込むコンテンツのIDを返している' do
      expect(@item.destination_id).to eq @panel.id
    end
  end
  
  describe 'クラスメソッドへのショートカットに於いて' do
    it 'binder_keyメソッドを呼び出せる' do
      expect(@item.binder_key).to eq @model.binder_key
    end
    
    it 'destination_keyメソッドを呼び出せる' do
      expect(@item.destination_key).to eq @model.destination_key
    end
    
  end
  
  describe 'playに於いて' do
    it '時系列でソートできるようなSQLを返している' do
      expect(@model.play_list_order).to eq 'scroll_panels.t'
    end
    
  end
  
  describe 'めんどいからチェックしない機能に於いて' do
    it 'とりあえず呼び出すところまでは確認しておく' do
      define_test @model
    end
    # max_t, find_t, 
    # own?, visible?
  end
  
    # new_t, collect_t, serial?, validate_t
    # store, destroy_and_shorten, 
    # insert_shift, lesser_shift, higher_shift, update_shift, rotate
end

describe SheetPanel do
  include LeafMacros
  before do
    @admin =FactoryGirl.create :admin
    @user_yas = FactoryGirl.create :user_yas_with_owner
    @user_rom = FactoryGirl.create :user_rom
    @author_yas =  @user_yas.author
    
    @model = ScrollPanel
    @scroll =  FactoryGirl.create :scroll_hello, author: @author_yas
    @panel =  FactoryGirl.create :panel_hello, author: @author_yas
    @item =  FactoryGirl.create :scroll_panel_plain, scroll: @scroll, panel: @panel
  end
  
  describe '生成メソッドに於いて' do
    before do
    end
    
    it 'ぶら下がるそれぞれのリーフを取得できる' do
      expect(@model.parent_model).to eq Scroll
    end
    it 'すべてのリーフを取得できる' do
      expect(@model.destination_model).to eq Panel
    end
    
    it 'ぶら下がるそれぞれのリーフを取得できる' do
      expect(@model.binder_key).to eq 'scroll_id'
    end
    
    it 'ぶら下がるそれぞれのリーフを取得できる' do
      expect(@model.destination_key).to eq 'panel_id'
    end
    
  end
  
  describe 'myに於いて' do
    before do
    end
    
    it '名前の活用に於いて' do
      expect(@item.binder_id).to eq @scroll.id
    end
    
    it '名前の活用に於いて' do
      expect(@item.destination_id).to eq @panel.id
    end
  end
  
  describe 'cutに於いて' do
    it 'cut' do
      expect(@item.binder_key).to eq @model.binder_key
    end
    
    it 'cut' do
      expect(@item.destination_key).to eq @model.destination_key
    end
    
  end
  
  describe 'playに於いて' do
    it 'cut' do
      expect(@model.play_list_order).to eq 'scroll_panels.t'
    end
    
  end
  
  describe 'めんどいからチェックしない機能に於いて' do
    it 'とりあえず呼び出すところまでは確認しておく' do
      define_test @model
    end
    # max_t, find_t, 
    # own?, visible?
  end
  
    # new_t, collect_t, serial?, validate_t
    # store, destroy_and_shorten, 
    # insert_shift, lesser_shift, higher_shift, update_shift, rotate
end

describe ComicStory do
  include LeafMacros
  before do
    @admin = FactoryGirl.create :admin
    @user_yas = FactoryGirl.create :user_yas_with_owner
    @user_rom = FactoryGirl.create :user_rom
    @author_yas = @user_yas.author
    
    @model = ComicStory
    @comic = FactoryGirl.create :comic_doc, author: @author_yas
    @story = FactoryGirl.create :story_install, author: @author_yas
    @item = FactoryGirl.create :comic_story_plain, comic: @comic, story: @story
  end
  
  describe '生成メソッドに於いて' do
    before do
    end
    
    it 'ぶら下がるそれぞれのリーフを取得できる' do
      expect(@model.parent_model).to eq Comic
    end
    it 'すべてのリーフを取得できる' do
      expect(@model.destination_model).to eq Story
    end
    
    it 'ぶら下がるそれぞれのリーフを取得できる' do
      expect(@model.binder_key).to eq 'comic_id'
    end
    
    it 'ぶら下がるそれぞれのリーフを取得できる' do
      expect(@model.destination_key).to eq 'story_id'
    end
    
  end
  
  describe 'myに於いて' do
    before do
    end
    
    it '名前の活用に於いて' do
      expect(@item.binder_id).to eq @comic.id
    end
    
    it '名前の活用に於いて' do
      expect(@item.destination_id).to eq @story.id
    end
  end
  
  describe 'cutに於いて' do
    it 'cut' do
      expect(@item.binder_key).to eq @model.binder_key
    end
    
    it 'cut' do
      expect(@item.destination_key).to eq @model.destination_key
    end
    
  end
  
  describe 'playに於いて' do
    it 'cut' do
      expect(@model.play_list_order).to eq 'comic_stories.t'
    end
    
  end
  
  describe 'めんどいからチェックしない機能に於いて' do
    it 'とりあえず呼び出すところまでは確認しておく' do
      define_test @model
    end
    # max_t, find_t, 
    # own?, visible?
  end
  
    # new_t, collect_t, serial?, validate_t
    # store, destroy_and_shorten, 
    # insert_shift, lesser_shift, higher_shift, update_shift, rotate
end

describe StorySheet do
  include LeafMacros
  before do
    @admin = FactoryGirl.create :admin
    @user_yas = FactoryGirl.create :user_yas_with_owner
    @user_rom = FactoryGirl.create :user_rom
    @author_yas =  @user_yas.author
    
    @model = StorySheet
    @story = FactoryGirl.create :story_install, author: @author_yas
    @sheet = FactoryGirl.create :sheet_install_a, author: @author_yas
    @item = FactoryGirl.create :story_sheet_plain, story: @story, sheet: @sheet
  end
  
  describe '生成メソッドに於いて' do
    before do
    end
    
    it 'ぶら下がるそれぞれのリーフを取得できる' do
      expect(@model.parent_model).to eq Story
    end
    it 'すべてのリーフを取得できる' do
      expect(@model.destination_model).to eq Sheet
    end
    
    it 'ぶら下がるそれぞれのリーフを取得できる' do
      expect(@model.binder_key).to eq 'story_id'
    end
    
    it 'ぶら下がるそれぞれのリーフを取得できる' do
      expect(@model.destination_key).to eq 'sheet_id'
    end
    
  end
  
  describe 'myに於いて' do
    before do
    end
    
    it '名前の活用に於いて' do
      expect(@item.binder_id).to eq @story.id
    end
    
    it '名前の活用に於いて' do
      expect(@item.destination_id).to eq @sheet.id
    end
  end
  
  describe 'cutに於いて' do
    it 'cut' do
      expect(@item.binder_key).to eq @model.binder_key
    end
    
    it 'cut' do
      expect(@item.destination_key).to eq @model.destination_key
    end
    
  end
  
  describe 'playに於いて' do
    it 'cut' do
      expect(@model.play_list_order).to eq 'story_sheets.t'
    end
    
  end
  
  describe 'めんどいからチェックしない機能に於いて' do
    it 'とりあえず呼び出すところまでは確認しておく' do
      define_test @model
    end
    # max_t, find_t, 
    # own?, visible?
  end
  
    # new_t, collect_t, serial?, validate_t
    # store, destroy_and_shorten, 
    # insert_shift, lesser_shift, higher_shift, update_shift, rotate
end

# I/O処理については、マニフェストで動作が変わることがないのでスクロールとスクコマでのみテストしておく
describe ScrollPanel do
  include LeafMacros
  before do
    @admin = FactoryGirl.create :admin
    @user_yas = FactoryGirl.create :user_yas_with_owner
    @user_rom = FactoryGirl.create :user_rom
    @author_yas = @user_yas.author
    
    @model = ScrollPanel
  end
  
  describe 't補充値new_tに於いて' do
    before do
      @dummy =  FactoryGirl.create :scroll_great_with_scroll_panels, author: @author_yas
      @scroll = FactoryGirl.create :scroll_hello, author: @author_yas
      @panel = FactoryGirl.create :panel_hello, author: @author_yas
    end
    
    context 'スクロール初のコマなら' do
      it '0を補充値とする' do
        expect(@model.new_t(@scroll.id)).to eq 0
      end
      
    end
    
    context 'スクロールにコマがあるとき' do
      it '一個なら1を補充値とする' do
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        expect(@model.new_t(@scroll.id)).to eq 1
      end
      
      it '3個なら3を補充値とする' do
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        @item =  FactoryGirl.create :scroll_panel_plain, t: 1, scroll: @scroll, panel: @panel
        @item =  FactoryGirl.create :scroll_panel_plain, t: 2, scroll: @scroll, panel: @panel
        expect(@model.new_t(@scroll.id)).to eq 3
      end
      
    end
    
  end
  
  describe 't収集collect_tに於いて' do
    before do
      @dummy =  FactoryGirl.create :scroll_great_with_scroll_panels, author: @author_yas
      @scroll =  FactoryGirl.create :scroll_hello, author: @author_yas
      @panel = FactoryGirl.create :panel_hello, author: @author_yas
    end
    
    context '空っぽのスクロールのとき' do
      it '当然リストは空っぽになっている' do
        expect(@model.collect_t(@scroll.id)).to be_empty
      end
      
    end
    
    context 'オーソドックスなデータのとき' do
      it 'スクコマから同一スクロールのtだけを収集している' do
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        expect(@model.collect_t(@scroll.id)).to eq [0]
      end
      
      it 'スクコマから同一スクロールのtだけを収集している' do
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        @item =  FactoryGirl.create :scroll_panel_plain, t: 1, scroll: @scroll, panel: @panel
        @item =  FactoryGirl.create :scroll_panel_plain, t: 2, scroll: @scroll, panel: @panel
        expect(@model.collect_t(@scroll.id)).to eq [0, 1, 2]
      end
      
    end
    
  end
  
  describe 'tチェックに於いて' do
    before do
      @dummy =  FactoryGirl.create :scroll_great_with_scroll_panels, author: @author_yas
      @scroll =  FactoryGirl.create :scroll_hello, author: @author_yas
      @panel = FactoryGirl.create :panel_hello, author: @author_yas
    end
    #実データで最低限のチェックを
    context '新規のとき' do
      it '一件だけで正常通過している' do
        @item =  FactoryGirl.build :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        r = ScrollPanel.validate_t @item
        expect(r).to be_true 
      end
    end
    context '既存のとき' do
      it '2件目を作っても正常通過している' do
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        @item2 =  FactoryGirl.build :scroll_panel_plain, t: 1, scroll: @scroll, panel: @panel
        r = ScrollPanel.validate_t @item2
        expect(r).to be_true 
      end
    end
  end
  describe '挿入シフトに於いて' do
    before do
      @dummy_scroll =  FactoryGirl.create :scroll_great_with_scroll_panels, author: @author_yas
      @scroll =  FactoryGirl.create :scroll_hello, author: @author_yas
      @panel = FactoryGirl.create :panel_hello, author: @author_yas
    end
    context 'テーブルに1件(t:0)で0に挿入したとき' do
      before do
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        @item2 =  FactoryGirl.build :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
      end
      it '既存の行を1にシフトしている' do
        @item2.insert_shift
        @item.reload
        expect(@item.t).to eq 1
      end
      it 'これから挿入するt(0)が欠番になっている' do
        @item2.insert_shift
        @item.reload
        expect(@item2.t).to eq 0
      end
    end
    context 'テーブルに2件(t:0,1)で1に挿入したとき' do
      before do
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        @item2 =  FactoryGirl.create :scroll_panel_plain, t: 1, scroll: @scroll, panel: @panel
        @item3 =  FactoryGirl.build :scroll_panel_plain, t: 1, scroll: @scroll, panel: @panel
      end
      it '既存のt1を2にシフトしてこれから挿入するt(1)が欠番になっている' do
        @item3.insert_shift
        @item.reload
        @item2.reload
        expect(@item.t).to eq 0
        expect(@item2.t).to eq 2
      end
    end
    context 'テーブルに5件(t:0,1,2,3,4)で2に挿入したとき' do
      before do
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        @item2 =  FactoryGirl.create :scroll_panel_plain, t: 1, scroll: @scroll, panel: @panel
        @item3 =  FactoryGirl.create :scroll_panel_plain, t: 2, scroll: @scroll, panel: @panel
        @item4 =  FactoryGirl.create :scroll_panel_plain, t: 3, scroll: @scroll, panel: @panel
        @item5 =  FactoryGirl.create :scroll_panel_plain, t: 4, scroll: @scroll, panel: @panel
        @item6 =  FactoryGirl.build :scroll_panel_plain, t: 2, scroll: @scroll, panel: @panel
      end
      it '既存のt1を2にシフトしてこれから挿入するt(1)が欠番になっている' do
        @item6.insert_shift
        @item.reload
        @item2.reload
        @item3.reload
        @item4.reload
        @item5.reload
        expect(@item.t).to eq 0
        expect(@item2.t).to eq 1
        expect(@item3.t).to eq 3
        expect(@item4.t).to eq 4
        expect(@item5.t).to eq 5
      end
    end
  end
  describe '少ない方に移動に於いて' do
    before do
      @scroll =  FactoryGirl.create :scroll_hello, author: @author_yas
      @panel = FactoryGirl.create :panel_hello, author: @author_yas
    end
    context 'テーブルに2件(t:0,1)で1を0に移動したとき' do
      before do
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        @item2 =  FactoryGirl.create :scroll_panel_plain, t: 1, scroll: @scroll, panel: @panel
        @ot = @item2.t
        @item2.t = 0
      end
      it '既存のt0を1にシフトしてこれから挿入するt(0)が欠番になっている' do
        #移動させたい行はそのまま残る
        @item2.lesser_shift @ot
        @item.reload
        @item2.reload
        expect(@item.t).to eq 1
        expect(@item2.t).to eq 1
      end
      it '既存のt0を1にシフトしている' do
        @item2.lesser_shift @ot
        @item.reload
        expect(@item.t).to eq 1
      end
    end
    context 'テーブルに3件(t:0,1,2)で2を1に移動したとき' do
      before do
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        @item2 =  FactoryGirl.create :scroll_panel_plain, t: 1, scroll: @scroll, panel: @panel
        @item3 =  FactoryGirl.create :scroll_panel_plain, t: 2, scroll: @scroll, panel: @panel
        @ot = @item3.t
        @item3.t = 1
      end
      it '既存のt1を2にシフトしてこれから挿入するt(1)が欠番になっている' do
        #移動させたい行はそのまま残る
        @item3.lesser_shift @ot
        @item.reload
        @item2.reload
        @item3.reload
        expect(@item.t).to eq 0
        expect(@item2.t).to eq 2
        expect(@item3.t).to eq 2
      end
      it '既存のt1を2にシフトしている' do
        @item3.lesser_shift @ot
        @item2.reload
        expect(@item2.t).to eq 2
      end
    end
    context 'テーブルに5件(t:0,1,2,3,4)で3を1に移動したとき' do
      before do
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        @item2 =  FactoryGirl.create :scroll_panel_plain, t: 1, scroll: @scroll, panel: @panel
        @item3 =  FactoryGirl.create :scroll_panel_plain, t: 2, scroll: @scroll, panel: @panel
        @item4 =  FactoryGirl.create :scroll_panel_plain, t: 3, scroll: @scroll, panel: @panel
        @item5 =  FactoryGirl.create :scroll_panel_plain, t: 4, scroll: @scroll, panel: @panel
        @ot = @item4.t
        @item4.t = 1
      end
      it 'これから挿入するt(1)が欠番になっている' do
        #移動させたい行はそのまま残る
        @item4.lesser_shift @ot
        l = ScrollPanel.find(:all).map {|s| s.t }
        l.sort.should eq [0, 2, 3, 3, 4]
      end
      it '既存のt0には変化がない' do
        @item4.lesser_shift @ot
        @item.reload
        @item.t.should eq 0
      end
      it '既存のt4には変化がない' do
        @item4.lesser_shift @ot
        @item5.reload
        @item5.t.should eq 4
      end
      it '既存のt1を2にシフトしている' do
        @item4.lesser_shift @ot
        @item2.reload
        @item2.t.should eq 2
      end
      it '既存のt2を3にシフトしている' do
        @item4.lesser_shift @ot
        @item3.reload
        @item3.t.should eq 3
      end
    end
    context '先ほどのケース+他のスクロール1件で挿入したとき' do
      before do
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        @item2 =  FactoryGirl.create :scroll_panel_plain, t: 1, scroll: @scroll, panel: @panel
        @item3 =  FactoryGirl.create :scroll_panel_plain, t: 2, scroll: @scroll, panel: @panel
        @item4 =  FactoryGirl.create :scroll_panel_plain, t: 3, scroll: @scroll, panel: @panel
        @item5 =  FactoryGirl.create :scroll_panel_plain, t: 4, scroll: @scroll, panel: @panel
        @ot = @item4.t
        @item4.t = 1
      end
      it 'これから挿入するt(1)が欠番になっている' do
        @item4.lesser_shift @ot
        l = ScrollPanel.find(:all).map {|s| s.t }
        l.sort.should eq [0, 0, 2, 3, 3, 4]
      end
      it '既存のt0には変化がない' do
        @item4.lesser_shift @ot
        @item.reload
        @item.t.should eq 0
      end
      it '既存のt4には変化がない' do
        @item4.lesser_shift @ot
        @item5.reload
        @item5.t.should eq 4
      end
      it '既存のt1を2にシフトしている' do
        @item4.lesser_shift @ot
        @item2.reload
        @item2.t.should eq 2
      end
      it '既存のt2を3にシフトしている' do
        @item4.lesser_shift @ot
        @item3.reload
        @item3.t.should eq 3
      end
      it '他のスクロールに影響がない' do
        @item4.lesser_shift @ot
        @itemc2.reload
        @itemc2.t.should eq 0
      end
    end
    #例外ケース。
    #負のときは0として正常扱い
    context 'テーブルに2件(t:0,1)で1を-1に移動したとき' do
      before do
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        @item = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @item2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @ot = @item2.t
        @item2.t = -1
      end
      it '既存のt0を1にシフトしてこれから挿入するt(0)が欠番になっている' do
        #移動させたい行はそのまま残る
        @item2.lesser_shift @ot
        l = ScrollPanel.find(:all).map {|s| s.t }
        l.sort.should eq [1, 1]
      end
      it '既存のt0を1にシフトしている' do
        @item2.lesser_shift @ot
        @item.reload
        @item.t.should eq 1
      end
      it '既存のt1は0に補正されている' do
        @item2.lesser_shift @ot
        @item2.t.should eq 0
      end
    end
  end
  describe '大きい方に移動に於いて' do
    before do
      @scroll =  FactoryGirl.create :scroll_hello, author: @author_yas
      @panel = FactoryGirl.create :panel_hello, author: @author_yas
    end
    context '依頼チェック' do
      it 'Updateを依頼している' do
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        @item = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @item2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        ScrollPanel.stub(:update_all).with(any_args)
        ScrollPanel.should_receive(:update_all).with(any_args).exactly(1)
        ot = @item.t
        @item.t = 1
        @item.higher_shift ot
      end
    end
    context 'テーブルに2件(t:0,1)で0を1に移動したとき' do
      before do
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        @item = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @item2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @ot = @item.t
        @item.t = 1
      end
      it '既存のt1を0にシフトしてこれから挿入するt(1)が欠番になっている' do
        #移動させたい行はそのまま残る
        @item.higher_shift @ot
        l = ScrollPanel.find(:all).map {|s| s.t }
        l.sort.should eq [0, 0]
      end
      it '既存のt1を0にシフトしている' do
        @item.higher_shift @ot
        @item2.reload
        @item2.t.should eq 0
      end
    end
    context 'テーブルに3件(t:0,1,2)で0を1に移動したとき' do
      before do
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        @item2 =  FactoryGirl.create :scroll_panel_plain, t: 1, scroll: @scroll, panel: @panel
        @item3 =  FactoryGirl.create :scroll_panel_plain, t: 2, scroll: @scroll, panel: @panel
        @ot = @item.t
        @item.t = 1
      end
      it '既存のt1を0にシフトしてこれから挿入するt(1)が欠番になっている' do
        #移動させたい行はそのまま残る
        @item.higher_shift @ot
        l = ScrollPanel.find(:all).map {|s| s.t }
        l.sort.should eq [0, 0, 2]
      end
      it '既存のt1を0にシフトしている' do
        @item.higher_shift @ot
        @item2.reload
        @item2.t.should eq 0
      end
    end
    context 'テーブルに5件(t:0,1,2,3,4)で1を3に移動したとき' do
      before do
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        @item2 =  FactoryGirl.create :scroll_panel_plain, t: 1, scroll: @scroll, panel: @panel
        @item3 =  FactoryGirl.create :scroll_panel_plain, t: 2, scroll: @scroll, panel: @panel
        @item4 =  FactoryGirl.create :scroll_panel_plain, t: 3, scroll: @scroll, panel: @panel
        @item5 =  FactoryGirl.create :scroll_panel_plain, t: 4, scroll: @scroll, panel: @panel
        @ot = @item2.t
        @item2.t = 3
      end
      it 'これから挿入するt(3)が欠番になっている' do
        #移動させたい行はそのまま残る
        @item2.higher_shift @ot
        l = ScrollPanel.find(:all).map {|s| s.t }
        l.sort.should eq [0, 1, 1, 2, 4]
      end
      it '既存のt0には変化がない' do
        @item2.higher_shift @ot
        @item.reload
        @item.t.should eq 0
      end
      it '既存のt4には変化がない' do
        @item2.higher_shift @ot
        @item5.reload
        @item5.t.should eq 4
      end
      it '既存のt2を1にシフトしている' do
        @item2.higher_shift @ot
        @item3.reload
        @item3.t.should eq 1
      end
      it '既存のt3を2にシフトしている' do
        @item2.higher_shift @ot
        @item4.reload
        @item4.t.should eq 2
      end
    end
    context '先ほどのケース+他のスクロール1件で挿入したとき' do
      before do
        @scroll2 = FactoryGirl.create :scroll, :author_id => @author.id
        @itemc2 = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll2.id, :panel_id => @panel.id, :author_id => @author.id
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        @item2 =  FactoryGirl.create :scroll_panel_plain, t: 1, scroll: @scroll, panel: @panel
        @item3 =  FactoryGirl.create :scroll_panel_plain, t: 2, scroll: @scroll, panel: @panel
        @item4 =  FactoryGirl.create :scroll_panel_plain, t: 3, scroll: @scroll, panel: @panel
        @item5 =  FactoryGirl.create :scroll_panel_plain, t: 4, scroll: @scroll, panel: @panel
        @ot = @item2.t
        @item2.t = 3
      end
      it 'これから挿入するt(3)が欠番になっている' do
        #移動させたい行はそのまま残る
        @item2.higher_shift @ot
        l = ScrollPanel.find(:all).map {|s| s.t }
        l.sort.should eq [0, 0, 1, 1, 2, 4]
      end
      it '既存のt0には変化がない' do
        @item2.higher_shift @ot
        @item.reload
        @item.t.should eq 0
      end
      it '既存のt4には変化がない' do
        @item2.higher_shift @ot
        @item5.reload
        @item5.t.should eq 4
      end
      it '既存のt2を1にシフトしている' do
        @item2.higher_shift @ot
        @item3.reload
        @item3.t.should eq 1
      end
      it '既存のt3を2にシフトしている' do
        @item2.higher_shift @ot
        @item4.reload
        @item4.t.should eq 2
      end
      it '他のスクロールに影響がない' do
        @item2.higher_shift @ot
        @itemc2.reload
        @itemc2.t.should eq 0
      end
    end
    #例外ケース。
    #max超えたときはmaxとして正常扱い
    context 'テーブルに2件(t:0,1)で0を2に移動したとき' do
      before do
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        @item = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @item2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @ot = @item.t
        @item.t = 2
      end
      it '既存のt1を0にシフトしてこれから挿入するt(1)が欠番になっている' do
        #移動させたい行はそのまま残る
        @item.higher_shift @ot
        l = ScrollPanel.find(:all).map {|s| s.t }
        l.sort.should eq [0, 0]
      end
      it '既存のt1を0にシフトしている' do
        @item.higher_shift @ot
        @item2.reload
        @item2.t.should eq 0
      end
      it '既存のt0は1に補正されている' do
        @item.higher_shift @ot
        @item.t.should eq 1
      end
    end
  end
  describe '入れ替えに於いて' do
    before do
      @scroll =  FactoryGirl.create :scroll_hello, author: @author_yas
      @panel = FactoryGirl.create :panel_hello, author: @author_yas
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
      @item = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
      @item2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
    end
    context '新tが旧tより小さいとき' do
      it '少ない方に移動を依頼している' do
        ScrollPanel.any_instance.stub(:lesser_shift).with(any_args)
        ScrollPanel.any_instance.should_receive(:lesser_shift).with(any_args).exactly(1)
        ot = @item2.t
        @item2.t = 0
        @item2.update_shift ot
      end
    end
    context '新tが旧tより大きいとき' do
      it '大きい方に移動を依頼している' do
        ScrollPanel.any_instance.stub(:higher_shift).with(any_args)
        ScrollPanel.any_instance.should_receive(:higher_shift).with(any_args).exactly(1)
        ot = @item.t
        @item.t = 1
        @item.update_shift ot
      end
    end
  end
  describe '順序入れ替えに於いて' do
    before do
      @scroll =  FactoryGirl.create :scroll_hello, author: @author_yas
      @panel = FactoryGirl.create :panel_hello, author: @author_yas
    end
    context 'オブジェクトが新規でtが空のとき' do
      it '末尾追加としてtを補充依頼している' do
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        @item = FactoryGirl.build :scroll_panel, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        ScrollPanel.stub(:new_t).with(any_args).and_return(0)
        ScrollPanel.should_receive(:new_t).with(any_args).exactly(1)
        @item.t = nil
        r = @item.rotate
      end
    end
    context 'オブジェクトが新規でtが設定されているとき' do
      it '挿入追加として挿入シフトを依頼している' do
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        @item = FactoryGirl.build :scroll_panel, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        ScrollPanel.any_instance.stub(:insert_shift).with(any_args)
        ScrollPanel.any_instance.should_receive(:insert_shift).with(any_args).exactly(1)
        @item.t = 0
        r = @item.rotate
      end
    end
    context 'オブジェクトが新規でなくtが設定されているとき' do
      it 'コマ移動として入れ替えを依頼している' do
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        @item = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @item2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        ScrollPanel.any_instance.stub(:update_shift).with(any_args)
        ScrollPanel.any_instance.should_receive(:update_shift).with(1).exactly(1)
        @item2.t = 0
        r = @item.rotate 1
      end
    end
    context 'オブジェクトが新規でなくtが空のとき' do
      it '入れ替えもシフトもせず、tを空のままにしている' do
        #結果、tに欠番が生じてシリアライズチェックでひっかかる
      end
    end
  end
  describe '保存に於いて' do
    before do
      @dummy_scroll =  FactoryGirl.create :scroll_great_with_scroll_panels, author: @author_yas
      @dummy_scroll_panels = @dummy_scroll.scroll_panels
      @scroll =  FactoryGirl.create :scroll_hello, author: @author_yas
      @panel = FactoryGirl.create :panel_hello, author: @author_yas
    end
    #入れ替えテストと同じテストを実施。こちらはシフトだけでなく本尊も更新されている
    context 'テーブルに5件(t:0,1,2,3,4)+他のスクロール1件で2に挿入したとき' do
      before do
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        @item2 =  FactoryGirl.create :scroll_panel_plain, t: 1, scroll: @scroll, panel: @panel
        @item3 =  FactoryGirl.create :scroll_panel_plain, t: 2, scroll: @scroll, panel: @panel
        @item4 =  FactoryGirl.create :scroll_panel_plain, t: 3, scroll: @scroll, panel: @panel
        @item5 =  FactoryGirl.create :scroll_panel_plain, t: 4, scroll: @scroll, panel: @panel
        @item6 =  FactoryGirl.build :scroll_panel_plain, t: 2, scroll: @scroll, panel: @panel
      end
      it '既存のt0には変化がない' do
        @item6.store
        @item.reload
        expect(@item.t).to eq 0
      end
      it '既存のt1には変化がない' do
        @item6.store
        @item2.reload
        expect(@item2.t).to eq 1
      end
      it '既存のt2を3にシフトしている' do
        @item6.store
        @item3.reload
        expect(@item3.t).to eq 3
      end
      it '既存のt3を4にシフトしている' do
        @item6.store
        @item4.reload
        expect(@item4.t).to eq 4
      end
      it '既存のt5を5にシフトしている' do
        @item6.store
        @item5.reload
        expect(@item5.t).to eq 5
      end
      it '新規のt2が作成されている' do
        @item6.store
        @item6.reload
        expect(@item6.t).to eq 2
      end
      it '他のスクロールに影響がない' do
        @before_status = @dummy_scroll_panels.map {|sp| [sp.id, sp.t]}
        @item6.store
        @after_status = @dummy_scroll_panels.map {|sp| sp.reload ; [sp.id, sp.t]}
        expect(@after_status).to eq @ot
      end
    end
    context 'テーブルに5件(t:0,1,2,3,4)+他のスクロール1件で3を1に移動したとき' do
      before do
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        @item2 =  FactoryGirl.create :scroll_panel_plain, t: 1, scroll: @scroll, panel: @panel
        @item3 =  FactoryGirl.create :scroll_panel_plain, t: 2, scroll: @scroll, panel: @panel
        @item4 =  FactoryGirl.create :scroll_panel_plain, t: 3, scroll: @scroll, panel: @panel
        @item5 =  FactoryGirl.create :scroll_panel_plain, t: 4, scroll: @scroll, panel: @panel
        @ot = @item4.t
        @item4.t = 1
      end
      it '既存のt0には変化がない' do
        @item4.store @ot
        @item.reload
        expect(@item.t).to eq 0
      end
      it '既存のt4には変化がない' do
        @item4.store @ot
        @item5.reload
        expect(@item5.t).to eq 4
      end
      it '既存のt1を2にシフトしている' do
        @item4.store @ot
        @item2.reload
        expect(@item2.t).to eq 2
      end
      it '既存のt2を3にシフトしている' do
        @item4.store @ot
        @item3.reload
        expect(@item3.t).to eq 3
      end
      it '既存のt3を1にシフトしている' do
        @item4.store @ot
        @item4.reload
        expect(@item4.t).to eq 1
      end
      it '他のスクロールに影響がない' do
        @before_status = @dummy_scroll_panels.map {|sp| [sp.id, sp.t]}
        @item4.store @ot
        @after_status = @dummy_scroll_panels.map {|sp| sp.reload ; [sp.id, sp.t]}
        expect(@after_status).to eq @ot
      end
    end
    context 'テーブルに5件(t:0,1,2,3,4)+他のスクロール1件で1を3に移動したとき' do
      before do
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        @item2 =  FactoryGirl.create :scroll_panel_plain, t: 1, scroll: @scroll, panel: @panel
        @item3 =  FactoryGirl.create :scroll_panel_plain, t: 2, scroll: @scroll, panel: @panel
        @item4 =  FactoryGirl.create :scroll_panel_plain, t: 3, scroll: @scroll, panel: @panel
        @item5 =  FactoryGirl.create :scroll_panel_plain, t: 4, scroll: @scroll, panel: @panel
        @ot = @item2.t
        @item2.t = 3
      end
      it '既存のt0には変化がない' do
        @item2.store @ot
        @item.reload
        expect(@item.t).to eq 0
      end
      it '既存のt4には変化がない' do
        @item2.store @ot
        @item5.reload
        expect(@item5.t).to eq 4
      end
      it '既存のt1を3にシフトしている' do
        @item2.store @ot
        @item2.reload
        expect(@item2.t).to eq 3
      end
      it '既存のt2を1にシフトしている' do
        @item2.store @ot
        @item3.reload
        expect(@item3.t).to eq 1
      end
      it '既存のt3を2にシフトしている' do
        @item2.store @ot
        @item4.reload
        expect(@item4.t).to eq 2
      end
      it '他のスクロールに影響がない' do
        @before_status = @dummy_scroll_panels.map {|sp| [sp.id, sp.t]}
        @item2.store @ot
        @after_status = @dummy_scroll_panels.map {|sp| sp.reload ; [sp.id, sp.t]}
        expect(@after_status).to eq @ot
      end
    end
    #ロールバックテスト。入れ替えが直接DBをいじるので、すべてのケースで確実にロールバックを確認する
    context 'テーブルに5件(t:0,1,2,3,4)+他のスクロール1件で2に挿入したが保存に失敗したとき' do
      before do
        ScrollPanel.any_instance.stub(:save).with(any_args).and_return(false)
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        @item2 =  FactoryGirl.create :scroll_panel_plain, t: 1, scroll: @scroll, panel: @panel
        @item3 =  FactoryGirl.create :scroll_panel_plain, t: 2, scroll: @scroll, panel: @panel
        @item4 =  FactoryGirl.create :scroll_panel_plain, t: 3, scroll: @scroll, panel: @panel
        @item5 =  FactoryGirl.create :scroll_panel_plain, t: 4, scroll: @scroll, panel: @panel
        @item6 = FactoryGirl.build :scroll_panel, :t => 2, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
      end
      it '既存のtに変化がない' do
        @item6.store
        @item.reload
        @item2.reload
        @item3.reload
        @item4.reload
        @item5.reload
        expect(@item.t).to eq 0
        expect(@item2.t).to eq 1
        expect(@item3.t).to eq 2
        expect(@item4.t).to eq 3
        expect(@item5.t).to eq 4
        expect(@item.t).to eq 5
        @itemc2.reload
        @itemc2.t.should eq 0
      end
      it 'falseを返す' do
        r = @item6.store
        expect(r).to be_false
      end
    end
    context 'テーブルに5件(t:0,1,2,3,4)+他のスクロール1件で3を1に移動したがシリアルチェックに失敗したとき' do
      before do
        ScrollPanel.stub(:validate_t).with(any_args).and_return(false)
        @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
        @item2 =  FactoryGirl.create :scroll_panel_plain, t: 1, scroll: @scroll, panel: @panel
        @item3 =  FactoryGirl.create :scroll_panel_plain, t: 2, scroll: @scroll, panel: @panel
        @item4 =  FactoryGirl.create :scroll_panel_plain, t: 3, scroll: @scroll, panel: @panel
        @item5 =  FactoryGirl.create :scroll_panel_plain, t: 4, scroll: @scroll, panel: @panel
        @ot = @item4.t
        @item4.t = 1
      end
      it '既存のtに変化がない' do
        @item4.store @ot
        @item.reload
        @item.t.should eq 0
        @item2.reload
        @item2.t.should eq 1
        @item3.reload
        @item3.t.should eq 2
        @item4.reload
        @item4.t.should eq 3
        @item5.reload
        @item5.t.should eq 4
        @itemc2.reload
        @itemc2.t.should eq 0
        expect(@item.t).to eq 0
        expect(@item.t).to eq 0
        expect(@item.t).to eq 0
        expect(@item.t).to eq 0
        expect(@item.t).to eq 0
        expect(@item.t).to eq 0
      end
      it 'falseを返す' do
        r = @item4.store @ot
        expect(r).to be_false
      end
      it 'tにエラーメッセージが入っている' do
        @item4.store @ot
        expect(@item4.errors[:t]).to be_empty
        expect(@item4.valid?).to be_true
      end
    end
  end
  describe '切り詰め処理つき削除に於いて' do
    before do
      @dummy =  FactoryGirl.create :scroll_great_with_scroll_panels, author: @author_yas
      @scroll =  FactoryGirl.create :scroll_hello, author: @author_yas
      @panel = FactoryGirl.create :panel_hello, author: @author_yas
      @item =  FactoryGirl.create :scroll_panel_plain, t: 0, scroll: @scroll, panel: @panel
    end
    context 'つつがなく終わるとき' do
      it '削除される' do
        expect {
          @item.destroy_and_shorten
        }.to change(ScrollPanel, :count ).by(-1)
      end
      it 'Trueを返す' do
        r = @item.destroy_and_shorten
        expect(r).to be_true 
      end
    end
    context '削除に失敗したとき' do
      before do
        ScrollPanel.any_instance.stub(:destroy).and_return(false)
      end
      it 'ロールバックされる' do
        expect {
          @item.destroy_and_shorten
        }.to change(ScrollPanel, :count )
      end
      it 'Falseを返す' do
        r = @item.destroy_and_shorten
        expect(r).to be_false
      end
    end
    #連携テスト。切り詰めが直接DBをいじる
    context '2件で先頭を削除したとき' do
      before do
        @item2 =  FactoryGirl.create :scroll_panel_plain, t: 1, scroll: @scroll, panel: @panel
      end
      it '行が削除される' do
        expect {
          @item.destroy_and_shorten
        }.to change(ScrollPanel, :count ).by(-1)
      end
      it '先頭は削除される' do
        @item.destroy_and_shorten
        expect {
          ScrollPanel.find @item.id
        }.to raise_error(ActiveRecord::RecordNotFound)
      end
      it '2件目は前に詰められる' do
        @item.destroy_and_shorten
        @item2.reload
        expect(@item2.t).to eq 0
      end
    end
    context '3件で先頭を削除したとき' do
      before do
        @item2 =  FactoryGirl.create :scroll_panel_plain, t: 1, scroll: @scroll, panel: @panel
        @item3 =  FactoryGirl.create :scroll_panel_plain, t: 2, scroll: @scroll, panel: @panel
      end
      it '行が削除される' do
        expect {
          @item.destroy_and_shorten
        }.to change(ScrollPanel, :count ).by(-1)
      end
      it '先頭は削除される' do
        @item.destroy_and_shorten
        expect {
          ScrollPanel.find @item.id
        }.to raise_error(ActiveRecord::RecordNotFound)
      end
      it '2件目は前に詰められる' do
        @item.destroy_and_shorten
        @item2.reload
        expect(@item2.t).to eq 0
      end
      it '3件目は前に詰められる' do
        @item.destroy_and_shorten
        @item3.reload
        expect(@item3.t).to eq 1
      end
    end
    context '5件で3件目を削除したとき' do
      before do
        @item2 =  FactoryGirl.create :scroll_panel_plain, t: 1, scroll: @scroll, panel: @panel
        @item3 =  FactoryGirl.create :scroll_panel_plain, t: 2, scroll: @scroll, panel: @panel
        @item4 =  FactoryGirl.create :scroll_panel_plain, t: 3, scroll: @scroll, panel: @panel
        @item5 =  FactoryGirl.create :scroll_panel_plain, t: 4, scroll: @scroll, panel: @panel
      end
      it '行が削除される' do
        expect {
          @item3.destroy_and_shorten
        }.to change(ScrollPanel, :count ).by(-1)
      end
      it '1件目は変化がない' do
        @item3.destroy_and_shorten
        @item.reload
        expect(@item.t).to eq 0
      end
      it '2件目は変化がない' do
        @item3.destroy_and_shorten
        @item2.reload
        expect(@item2.t).to eq 1
      end
      it '3件目は削除される' do
        @item3.destroy_and_shorten
        expect {
          ScrollPanel.find @item3.id
        }.to raise_error(ActiveRecord::RecordNotFound)
      end
      it '4件目は前に詰められる' do
        @item3.destroy_and_shorten
        @item4.reload
        expect(@item4.t).to eq 2
      end
      it '5件目は前に詰められる' do
        @item3.destroy_and_shorten
        @item5.reload
        expect(@item5.t).to eq 3
      end
    end
    #ロールバックテスト。切り詰めが直接DBをいじるので、すべてのケースで確実にロールバックを確認する
  end
end
